/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory for Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */

#include "zeroLocal2c2eIntegrals.h"

namespace Scine {
namespace Sparrow {

namespace nddo {
using namespace GeneralTypes;

namespace multipole {

bool ZeroLocal2c2eIntegrals::isZero(GeneralTypes::twoElIntegral_t t1, GeneralTypes::twoElIntegral_t t2) {
  // Will be initialized on the first call
  static Array zeroElements = getArrayForZeroIntegrals();

  auto index1 = static_cast<int>(t1);
  auto index2 = static_cast<int>(t2);
  return zeroElements[index1][index2];
}

void ZeroLocal2c2eIntegrals::setElementToNonZero(ZeroLocal2c2eIntegrals::Array& array, GeneralTypes::twoElIntegral_t t1,
                                                 GeneralTypes::twoElIntegral_t t2) {
  auto index1 = static_cast<int>(t1);
  auto index2 = static_cast<int>(t2);
  array[index1][index2] = false;
  array[index2][index1] = false;
}

ZeroLocal2c2eIntegrals::Array ZeroLocal2c2eIntegrals::getArrayForZeroIntegrals() {
  Array array;

  for (int i = 0; i < numberTwoElectronIntegrals; ++i) {
    for (int j = 0; j < numberTwoElectronIntegrals; ++j) {
      array[i][j] = true;
    }
  }

  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_s, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_x, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::s_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::z2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_x, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::s_y, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_y, twoElIntegral_t::x_y);
  setElementToNonZero(array, twoElIntegral_t::x_y, twoElIntegral_t::s_xy);
  setElementToNonZero(array, twoElIntegral_t::x_y, twoElIntegral_t::z2_xy);
  setElementToNonZero(array, twoElIntegral_t::x_y, twoElIntegral_t::xz_yz);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::s_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::z2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_y, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_z, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_z, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::y_z, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z_z, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_z2, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_xz, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::s_yz, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_x2y2, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::s_x2y2, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::s_x2y2, twoElIntegral_t::s_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_x2y2, twoElIntegral_t::z2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::s_x2y2, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::s_x2y2, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::s_xy, twoElIntegral_t::x_y);
  setElementToNonZero(array, twoElIntegral_t::s_xy, twoElIntegral_t::s_xy);
  setElementToNonZero(array, twoElIntegral_t::s_xy, twoElIntegral_t::z2_xy);
  setElementToNonZero(array, twoElIntegral_t::s_xy, twoElIntegral_t::xz_yz);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_z2, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_xz, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_x2y2, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::x_xy, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::y_z2, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_yz, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::y_x2y2, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::y_xy, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z_z2, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z_xz, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::z_yz, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z2_z2, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z2_xz, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::z2_yz, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z2_x2y2, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::z2_x2y2, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::z2_x2y2, twoElIntegral_t::s_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z2_x2y2, twoElIntegral_t::z2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::z2_x2y2, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::z2_x2y2, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::z2_xy, twoElIntegral_t::x_y);
  setElementToNonZero(array, twoElIntegral_t::z2_xy, twoElIntegral_t::s_xy);
  setElementToNonZero(array, twoElIntegral_t::z2_xy, twoElIntegral_t::z2_xy);
  setElementToNonZero(array, twoElIntegral_t::z2_xy, twoElIntegral_t::xz_yz);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::s_x2y2);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::z2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::xz_xz, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::xz_yz, twoElIntegral_t::x_y);
  setElementToNonZero(array, twoElIntegral_t::xz_yz, twoElIntegral_t::s_xy);
  setElementToNonZero(array, twoElIntegral_t::xz_yz, twoElIntegral_t::z2_xy);
  setElementToNonZero(array, twoElIntegral_t::xz_yz, twoElIntegral_t::xz_yz);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::xz_x2y2, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::xz_xy, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::s_x2y2);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::z2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::yz_yz, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::s_y);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::y_z);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::s_yz);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::x_xy);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::y_z2);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::y_x2y2);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::z_yz);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::z2_yz);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::xz_xy);
  setElementToNonZero(array, twoElIntegral_t::yz_x2y2, twoElIntegral_t::yz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::s_x);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::x_z);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::s_xz);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::x_z2);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::x_x2y2);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::y_xy);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::z_xz);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::z2_xz);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::xz_x2y2);
  setElementToNonZero(array, twoElIntegral_t::yz_xy, twoElIntegral_t::yz_xy);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::x2y2_x2y2, twoElIntegral_t::xy_xy);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::s_s);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::x_x);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::y_y);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::s_z);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::z_z);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::s_z2);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::x_xz);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::y_yz);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::z_z2);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::z2_z2);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::xz_xz);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::yz_yz);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::x2y2_x2y2);
  setElementToNonZero(array, twoElIntegral_t::xy_xy, twoElIntegral_t::xy_xy);

  return array;
}

} // namespace multipole

} // namespace nddo
} // namespace Sparrow
} // namespace Scine
